#include "abstract-dma.hpp"

using namespace stm32f072xb;

AbstractDma::AbstractDma(DMA_Channel_TypeDef* dmaChannel, AbstractDmaInit& dmaInit)
    : _dmaChannel(dmaChannel), _dmaInit(dmaInit) {
	if (!READ_BIT(RCC->AHBENR, RCC_AHBENR_DMAEN)) {
		SET_BIT(RCC->AHBENR, RCC_AHBENR_DMAEN);
	}
	setupRegister();
}

AbstractDma::~AbstractDma() {
	stop();
}

void AbstractDma::start() {
	if (READ_BIT(_dmaChannel->CCR, DMA_CCR_EN)) {
		stop();
		setNumberOfDataTransfer(_dmaInit.bufferSize);
	}
	SET_BIT(_dmaChannel->CCR, DMA_CCR_EN);
}

void AbstractDma::stop() {
	CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_EN);
}

void AbstractDma::setupRegister() {
	WRITE_REG(_dmaChannel->CCR, 0);
	setPeripheral(_dmaInit.peripheral);
	setMemory(_dmaInit.buffer);
	setNumberOfDataTransfer(_dmaInit.bufferSize);
	setDirection(_dmaInit.direction);
	setCircularMode(_dmaInit.circularMode);
	setMemoryIncrement(_dmaInit.memoryIncrement);
	setMemorySize(_dmaInit.memorySize);
	setPeripheralIncrement(_dmaInit.peripheralIncrement);
	setPeripheralSize(_dmaInit.peripheralSize);
}

void AbstractDma::setPeripheral(DmaPeripheral peripheral) {
	switch (peripheral) {
		case DmaPeripheral::USART2_RX:
			_dmaChannel->CPAR = (uintptr_t)&USART2->RDR;
			break;
		case DmaPeripheral::USART2_TX:
			_dmaChannel->CPAR = (uintptr_t)&USART2->TDR;
			break;
		case DmaPeripheral::I2C1_RX:
			_dmaChannel->CPAR = (uintptr_t)&I2C1->RXDR;
			break;
		case DmaPeripheral::I2C1_TX:
			_dmaChannel->CPAR = (uintptr_t)&I2C1->TXDR;
			break;
	}
}

void AbstractDma::setMemory(volatile const uint8_t* buffer) {
	_dmaChannel->CMAR = (uintptr_t)(buffer);
}

void AbstractDma::setNumberOfDataTransfer(uint16_t numberOfDataTransfer) {
	WRITE_REG(_dmaChannel->CNDTR, numberOfDataTransfer);
}

void AbstractDma::setDirection(DmaDirection direction) {
	switch (direction) {
		case DmaDirection::MEMORY_TO_PERIPHERAL:
			SET_BIT(_dmaChannel->CCR, DMA_CCR_DIR);
			break;
		case DmaDirection::PERIPHERAL_TO_MEMORY:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_DIR);
			break;
	}
}

void AbstractDma::setCircularMode(DmaCircularMode circularMode) {
	switch (circularMode) {
		case DmaCircularMode::ENABLE:
			SET_BIT(_dmaChannel->CCR, DMA_CCR_CIRC);
			break;
		case DmaCircularMode::DISABLE:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_CIRC);
			break;
	}
}

void AbstractDma::setMemoryIncrement(DmaMemoryIncrement memoryIncrement) {
	switch (memoryIncrement) {
		case DmaMemoryIncrement::ENABLE:
			SET_BIT(_dmaChannel->CCR, DMA_CCR_MINC);
			break;
		case DmaMemoryIncrement::DISABLE:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_MINC);
			break;
	}
}

void AbstractDma::setMemorySize(DmaMemorySize memorySize) {
	switch (memorySize) {
		case DmaMemorySize::BYTE:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_MSIZE);
			break;
		case DmaMemorySize::HALF_WORD:
			MODIFY_REG(_dmaChannel->CCR, DMA_CCR_PSIZE, DMA_CCR_MSIZE_0);
			break;
		case DmaMemorySize::WORD:
			MODIFY_REG(_dmaChannel->CCR, DMA_CCR_PSIZE, DMA_CCR_MSIZE_1);
			break;
	}
}

void AbstractDma::setPeripheralIncrement(DmaPeripheralIncrement peripheralIncrement) {
	switch (peripheralIncrement) {
		case DmaPeripheralIncrement::ENABLE:
			SET_BIT(_dmaChannel->CCR, DMA_CCR_PINC);
			break;
		case DmaPeripheralIncrement::DISABLE:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_PINC);
			break;
	}
}

void AbstractDma::setPeripheralSize(DmaPeripheralSize peripheralSize) {
	switch (peripheralSize) {
		case DmaPeripheralSize::BYTE:
			CLEAR_BIT(_dmaChannel->CCR, DMA_CCR_PSIZE);
			break;
		case DmaPeripheralSize::HALF_WORD:
			MODIFY_REG(_dmaChannel->CCR, DMA_CCR_PSIZE, DMA_CCR_PSIZE_0);
			break;
		case DmaPeripheralSize::WORD:
			MODIFY_REG(_dmaChannel->CCR, DMA_CCR_PSIZE, DMA_CCR_PSIZE_1);
			break;
	}
}
